import re
from typing import List


def _clean(rg: str) -> str:
    return re.sub(r'\D', '', rg)


def _is_allowed(rg: str) -> bool:
    first_digit = rg[0]
    return not all(first_digit == digit for digit in rg)


def _generate_factories(rg: str) -> List[int]:
    return list(map(lambda i: i if i <= 9 else i - 8, range(2, len(rg) + 1)))


def _caculate_digit(rg: str) -> str:
    FACTORIES = _generate_factories(rg)
    rg = list(map(lambda x: int(x), rg[:9]))
    result_sum = sum([rg[index] * factory for index, factory in enumerate(FACTORIES)])
    REST = result_sum % 11
    DV = str(11 - REST)
    if DV == '10':
        return 'X'
    if DV == '11':
        return '0'
    return DV


def is_valid(rg: str) -> bool:
    rg: str = _clean(rg)
    if not _is_allowed(rg):
        return False
    return rg[-1] == _caculate_digit(rg)


# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['validate_rg']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'validate-rg',
    'version': '0.0.1',
    'description': 'Faz a validação de RG (SSP SP) Brasileiro',
    'long_description': "# validate-rg\n\n[![pipy](https://img.shields.io/pypi/v/validate_rg.svg)](https://pypi.python.org/pypi/validate_rg)\n\nValidates Brazilian RG (SSP SP)\n\n## Features\n\n- RG Validation with mask\n- RG Validation without mask\n\n## Modes of use\n\n```python\n#!/usr/bin/python\nimport validate_rg\n\n# Without mask\nvalidate_rg.is_valid('505675092') # True\n\n# With mask\nvalidate_rg.is_valid('50.567.509-2') # True\n```\n\nor\n\n```python\n#!/usr/bin/python\nfrom validate_rg import is_valid\n\n# Without mask\nis_valid('111111111') # False\n\n# With mask\nis_valid('11.111.111-1') # False\n```\n\n# Author\n\n[João Filho](https://joaofilho.dev)\n[Github](https://github.com/drummerzzz)\n\n# Credits\n\nThis package was created with Cookiecutter and the `cs01/cookiecutter-pypackage` project template.\n\n[Cookiecutter](https://github.com/audreyr/cookiecutter)\n\n[cs01/cookiecutter-pypackage](https://github.com/cs01/cookiecutter-pypackage)\n\n[DV calculation](https://www.ngmatematica.com/2014/02/como-determinar-o-digito-verificador-do.html)",
    'author': 'Drummerzzz',
    'author_email': 'devdrummerzzz@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/drummerzzz/pypi_validate_rg/README.md',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
